# this forge file is for site specific tweaks
import os
import sys

forge = sys.modules["forge"]

def local_setup():
	# allow OpenMP
	# OpenMP uses up "static TLS" (if you run out, consider setting this False)
	forge.omp = True

	# parent path for the various versions of Houdini
	# (overriden if the HT environment variable is found)
	forge.houdini_installs = '/opt/'

	# default paths for FE Houdini nodes
	forge.houdini_url_root = 'http://freeelectron.org/op/'
	forge.houdini_icon_root = 'http://freeelectron.org/svg/'

	# desired potential Houdini versions
	forge.houdini_versions = [
			'11.0.775',
			'12.0.581',
			'12.1.125',
			'12.5.316.22',
			'12.5.376' ]

	# mapping from Boost compiler string to actual compiler command
	# the default compiler does not need to be listed
	forge.compilers = {
			'gcc41': 'g++-4.1' }

	# other Boost versions as [version, lib_suffix, include, lib]
	# version is like 1_46, usually blank for the distribution default
	# lib_suffix can be something like '-gcc43-mt-1_51', if needed
	# the distribution lib is usually in /usr/lib or /usr/lib64
	forge.boost_sources = []
	if os.path.exists('/usr/include/boost/exception'):
		forge.cppmap['boost'] = '-DFE_BOOST_EXCEPTIONS'
		forge.boost_sources += [ [ '', '', '/usr/include', '/usr/lib' ] ]

	# TBB (see boost_sources)
	forge.tbb_sources = []
	if os.path.exists('/usr/include/tbb/compat/condition_variable'):
		forge.tbb_sources = [ [ '', '', '/usr/include', '/usr/lib' ] ]

	# Alembic (see boost_sources)
	forge.alembic_sources = []

	# OpenEXR fallback path (if not adjacent to Alembic)
	forge.openexr_source = ''

	# parent path for the various versions of Maya
	# (overriden if the MAYA_ROOT environment variable is found)
	forge.maya_installs = '/usr/autodesk/'

	# desired potential Maya versions
	forge.maya_versions = [
			'2012',
			'2013',
			'2014' ]

	# parent path to "include/OpenImageIO/imageio.h" & "lib/libOpenImageIO.so"
	# leave blank to not use OpenImageIO
	forge.oiio_dist = 'ext/oiio/oiio-RB-1.2/dist/linux64'

	# parent paths to "openvdb.h" and "libopenvdb.so"
	# leave blank to not use OpenVDB
	forge.openvdb_include = 'ext/vdb/openvdb'
	forge.openvdb_lib = forge.openvdb_include

	# default order to search for compatible implementations
	# 'omp' specifies direct OpenMP (not an FE dynamic library)
	forge.regex_order = [
			'fexPcre',
			'fexBoostRegex1_51',
			'fexBoostRegex1_46',
			'fexBoostRegex1_37',
			'fexBoostRegex' ]
	forge.thread_order = [
			'fexTbb6105',
			'fexTbb6002',
			'fexTbb5000',
			'fexTbb',
			'fexBoostThread1_51',
			'fexBoostThread1_46',
			'fexBoostThread1_37',
			'fexBoostThread',
			'omp'
			]
	forge.mutex_order = forge.thread_order

	# which operator libraries to use
	# 'replace' is a new starting list; leave blank to configure automatically
	forge.houdini_oplibs_replace = []
	forge.maya_oplibs_replace = []
	# 'append' is added to the list, perhaps locally created libraries
	forge.houdini_oplibs_append = []
	forge.maya_oplibs_append = []

